import type { ComponentChild } from 'preact';
import type { DeepPartial } from 'ts-essentials';
import EventModel from "../model/eventModel";
import TZDate from "../time/date";
import type { EventBus } from "../utils/eventBus";
import type { ExternalEventTypes } from "../types/eventBus";
import type { DateType, EventObject } from "../types/events";
import type { CalendarColor, CalendarInfo, Options, ViewType } from "../types/options";
import type { CalendarState, CalendarStore, Dispatchers, InternalStoreAPI } from "../types/store";
import type { ThemeState, ThemeStore } from "../types/theme";
export default abstract class CalendarControl implements EventBus<ExternalEventTypes> {
    protected container: Element | null;
    /**
     * start and end date of weekly, monthly
     * @type {object}
     * @private
     */
    protected renderRange: {
        start: TZDate;
        end: TZDate;
    };
    protected eventBus: EventBus<ExternalEventTypes>;
    protected theme: InternalStoreAPI<ThemeStore>;
    protected store: InternalStoreAPI<CalendarStore>;
    constructor(container: string | Element, options?: Options);
    protected abstract getComponent(): ComponentChild;
    protected getStoreState(): CalendarState;
    protected getStoreState<Group extends keyof CalendarState>(group: Group): CalendarState[Group];
    protected getStoreDispatchers(): Dispatchers;
    protected getStoreDispatchers<Group extends keyof Dispatchers>(group: Group): Dispatchers[Group];
    /**
     * Destroys the instance.
     */
    destroy(): void;
    private calculateMonthRenderDate;
    private calculateWeekRenderDate;
    private calculateDayRenderDate;
    /**
     * Move the rendered date to the next/prev range.
     *
     * The range of movement differs depending on the current view, Basically:
     *   - In month view, it moves to the next/prev month.
     *   - In week view, it moves to the next/prev week.
     *   - In day view, it moves to the next/prev day.
     *
     * Also, the range depends on the options like how many visible weeks/months should be rendered.
     *
     * @param {number} offset The offset to move by.
     *
     * @example
     * // Move to the next month in month view.
     * calendar.move(1);
     *
     * // Move to the next year in month view.
     * calendar.move(12);
     *
     * // Move to yesterday in day view.
     * calendar.move(-1);
     */
    move(offset: number): void;
    /**********
     * CRUD Methods
     **********/
    /**
     * Create events and render calendar.
     * @param {EventObject[]} events - list of {@link EventObject}
     * @example
     * calendar.createEvents([
     *   {
     *     id: '1',
     *     calendarId: '1',
     *     title: 'my event',
     *     category: 'time',
     *     dueDateClass: '',
     *     start: '2018-01-18T22:30:00+09:00',
     *     end: '2018-01-19T02:30:00+09:00',
     *   },
     *   {
     *     id: '2',
     *     calendarId: '1',
     *     title: 'second event',
     *     category: 'time',
     *     dueDateClass: '',
     *     start: '2018-01-18T17:30:00+09:00',
     *     end: '2018-01-19T17:31:00+09:00',
     *   },
     * ]);
     */
    createEvents(events: EventObject[]): void;
    protected getEventModel(eventId: string, calendarId: string): EventModel | null;
    /**
     * Get an {@link EventObject} with event's id and calendar's id.
     *
     * @param {string} eventId - event's id
     * @param {string} calendarId - calendar's id of the event
     * @returns {EventObject | null} event. If the event can't be found, it returns null.
     *
     * @example
     * const event = calendar.getEvent(eventId, calendarId);
     *
     * console.log(event.title);
     */
    getEvent(eventId: string, calendarId: string): import("../types/events").EventObjectWithDefaultValues | null;
    /**
     * Update an event.
     *
     * @param {string} eventId - ID of an event to update
     * @param {string} calendarId - The calendarId of the event to update
     * @param {EventObject} changes - The new {@link EventObject} data to apply to the event
     *
     * @example
     * calendar.on('beforeUpdateEvent', function ({ event, changes }) {
     *   const { id, calendarId } = event;
     *
     *   calendar.updateEvent(id, calendarId, changes);
     * });
     */
    updateEvent(eventId: string, calendarId: string, changes: EventObject): void;
    /**
     * Delete an event.
     *
     * @param {string} eventId - event's id to delete
     * @param {string} calendarId - The CalendarId of the event to delete
     */
    deleteEvent(eventId: string, calendarId: string): void;
    /**********
     * General Methods
     **********/
    /**
     * Set events' visibility by calendar ID
     *
     * @param {string|string[]} calendarId - The calendar id or ids to change visibility
     * @param {boolean} isVisible - If set to true, show the events. If set to false, hide the events.
     */
    setCalendarVisibility(calendarId: string | string[], isVisible: boolean): void;
    /**
     * Render the calendar.
     *
     * @example
     * calendar.render();
     *
     * @example
     * // Re-render the calendar when resizing a window.
     * window.addEventListener('resize', () => {
     *   calendar.render();
     * });
     */
    render(): this;
    /**
     * For SSR(Server Side Rendering), Return the HTML string of the whole calendar.
     *
     * @returns {string} HTML string
     */
    renderToString(): string;
    /**
     * Delete all events and clear view
     *
     * @example
     * calendar.clear();
     */
    clear(): void;
    /**
     * TODO: implement this
     * Scroll to current time on today in case of daily, weekly view.
     *
     * @example
     * function onNewEvents(events) {
     *     calendar.createEvents(events);
     *     if (calendar.getViewName() !== 'month') {
     *         calendar.scrollToNow();
     *     }
     * }
     */
    scrollToNow(): void;
    /**
     * Move to today.
     *
     * @example
     * function onClickTodayBtn() {
     *   calendar.today();
     * }
     */
    today(): void;
    /**
     * Move to specific date.
     *
     * @param {DateType} date - The date to move
     * @example
     * calendar.on('clickDayname', (event) => {
     *   if (calendar.getViewName() === 'week') {
     *     const dateToMove = new Date(event.date);
     *
     *     calendar.setDate(dateToMove);
     *     calendar.changeView('day');
     *   }
     * });
     */
    setDate(date: DateType): void;
    /**
     * Move the calendar forward to the next range.
     *
     * @example
     * function moveToNextOrPrevRange(offset) {
     *   if (offset === -1) {
     *     calendar.prev();
     *   } else if (offset === 1) {
     *     calendar.next();
     *   }
     * }
     */
    next(): void;
    /**
     * Move the calendar backward to the previous range.
     *
     * @example
     * function moveToNextOrPrevRange(offset) {
     *   if (offset === -1) {
     *     calendar.prev();
     *   } else if (offset === 1) {
     *     calendar.next();
     *   }
     * }
     */
    prev(): void;
    /**
     * Change color values of events belong to a certain calendar.
     *
     * @param {string} calendarId - The calendar ID
     * @param {CalendarColor} colorOptions - The {@link CalendarColor} object
     *
     * @example
     * calendar.setCalendarColor('1', {
     *     color: '#e8e8e8',
     *     backgroundColor: '#585858',
     *     borderColor: '#a1b56c',
     *     dragBackgroundColor: '#585858',
     * });
     * calendar.setCalendarColor('2', {
     *     color: '#282828',
     *     backgroundColor: '#dc9656',
     *     borderColor: '#a1b56c',
     *     dragBackgroundColor: '#dc9656',
     * });
     * calendar.setCalendarColor('3', {
     *     color: '#a16946',
     *     backgroundColor: '#ab4642',
     *     borderColor: '#a1b56c',
     *     dragBackgroundColor: '#ab4642',
     * });
     */
    setCalendarColor(calendarId: string, colorOptions: CalendarColor): void;
    /**
     * Change current view type.
     *
     * @param {ViewType} viewName - The new view name to change
     *
     * @example
     * // change to daily view
     * calendar.changeView('day');
     *
     * // change to weekly view
     * calendar.changeView('week');
     *
     * // change to monthly view
     * calendar.changeView('month');
     */
    changeView(viewName: ViewType): void;
    /**
     * Get the DOM element of the event by event id and calendar id
     *
     * @param {string} eventId - ID of event
     * @param {string} calendarId - calendarId of event
     * @returns {HTMLElement} event element if found or null
     *
     * @example
     * const element = calendar.getElement(eventId, calendarId);
     *
     * console.log(element);
     */
    getElement(eventId: string, calendarId: string): Element | null;
    /**
     * Set the theme of the calendar.
     *
     * @param {DeepPartial<ThemeState>} theme - theme object
     *
     * @example
     * calendar.setTheme({
     *   common: {
     *     gridSelection: {
     *       backgroundColor: '#333',
     *     },
     *   },
     *   week: {
     *     currentTime: {
     *       color: '#00FF00',
     *     },
     *   },
     *   month: {
     *     dayname: {
     *       borderLeft: '1px solid #e5e5e5',
     *     },
     *   },
     * });
     */
    setTheme(theme: DeepPartial<ThemeState>): void;
    /**
     * Get current {@link Options}.
     *
     * @returns {Options} options
     */
    getOptions(): {
        template: import("../types/template").Template;
        theme: {
            common: import("../types/theme").CommonTheme;
            week: import("../types/theme").WeekTheme;
            month: import("../types/theme").MonthTheme;
        };
        month: import("../types/options").MonthOptions;
        week: import("../types/options").WeekOptions;
        isReadOnly: boolean;
        gridSelection: (boolean | import("../types/options").GridSelectionOptions) & import("../types/options").GridSelectionOptions;
        defaultView: ViewType;
        useFormPopup: boolean;
        useDetailPopup: boolean;
        usageStatistics: boolean;
        eventFilter: (event: EventObject) => boolean;
        timezone: import("../types/options").TimezoneOptions;
    };
    /**
     * Set options of calendar.
     *
     * @param {Options} options - set {@link Options}
     */
    setOptions({ theme, template, ...restOptions }: Options): void;
    /**
     * Get current rendered date.
     *
     * @returns {TZDate}
     */
    getDate(): TZDate;
    /**
     * Start time of rendered date range. ({@link TZDate} for further information)
     *
     * @returns {TZDate}
     */
    getDateRangeStart(): TZDate;
    /**
     * End time of rendered date range. ({@link TZDate} for further information)
     *
     * @returns {TZDate}
     */
    getDateRangeEnd(): TZDate;
    /**
     * Get current view name('day', 'week', 'month').
     *
     * @returns {ViewType} current view name
     */
    getViewName(): ViewType;
    /**
     * Set calendar list.
     *
     * @param {CalendarInfo[]} calendars - list of calendars
     */
    setCalendars(calendars: CalendarInfo[]): void;
    /**
     * TODO: specify position of popup
     *
     * Open event form popup with predefined form values.
     *
     * @param {EventObject} event - The preset {@link EventObject} data
     */
    openFormPopup(event: EventObject): void;
    fire<EventName extends keyof ExternalEventTypes>(eventName: EventName, ...args: Parameters<ExternalEventTypes[EventName]>): EventBus<ExternalEventTypes>;
    off<EventName extends keyof ExternalEventTypes>(eventName?: EventName, handler?: ExternalEventTypes[EventName]): EventBus<ExternalEventTypes>;
    on<EventName extends keyof ExternalEventTypes>(eventName: EventName, handler: ExternalEventTypes[EventName]): EventBus<ExternalEventTypes>;
    once<EventName extends keyof ExternalEventTypes>(eventName: EventName, handler: ExternalEventTypes[EventName]): EventBus<ExternalEventTypes>;
}
